// BlockArg.as
// John Maloney, August 2009
//
// A BlockArg represents a Block argument slot. Some BlockArgs, contain
// a text field that can be edited by the user. Others (e.g. booleans)
// are immutable. In either case, they be replaced by a reporter block
// of the right type. That is, dropping a reporter block onto a BlockArg
// inside a block causes the BlockArg to be replaced by the reporter.
// If a reporter is removed, a BlockArg is added to the block.

package Blocks {
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.filters.BevelFilter;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFieldType;
	import flash.text.TextFormat;
	import mx.core.UIComponent;

public class BlockArg extends UIComponent {

	public var type:String;
	public var base:BlockShape;
	public var argValue:*;
	private var field:TextField;

	// BlockArg types:
	//	b - boolean (pointed)
	//  n - number (rounded)
	//  s - string (rectangular)
	public function BlockArg(type:String, color:int, editable:Boolean = false) {
		var c:int = Color.scaleBrightness(color, 0.85);
		this.type = type;
		if (type == "b") {
			base = new BlockShape(Shapes.BooleanShape, c);
			argValue = false;
		} else if (type == "n") {
			base = new BlockShape(Shapes.NumberShape, c);
			argValue = 0;
		} else {
			base = new BlockShape(Shapes.RectShape, c);
			argValue = "";
		}

		base.setWidthHeight(30, 15);

		// bevel
		if (base.filters.length > 0) {
			var f:BevelFilter = BevelFilter(base.filters[0]);
			f.angle = 240;
			base.filters = [f];
		}
		width = base.width;
		height = base.height;
		scaleX = scaleY = 1;
		addChild(base);

		if (editable) { // add an editable string field and set color to white
			field = makeTextField();
			field.text = "abc";
			field.x = 2;
			field.y = -1;
			if (type == "n") {
				field.text = "10";
				field.x = 3;
				field.y = 0;
			}
			base.setColor(0xFFFFFF);
			addChild(field);
			textChanged(null);
		}
	}

	public function setDefault(value:*):void {
		if (field != null) {
			field.text = value;
			textChanged(null);
		}
	}

	private function makeTextField():TextField {
		var format:TextFormat = new TextFormat();
		format.color = 0;
		format.font = "Verdana";
		format.size = 10;

		var tf:TextField = new TextField();
		tf.autoSize = TextFieldAutoSize.LEFT;
		tf.defaultTextFormat = format;
		tf.type = TextFieldType.INPUT;

		tf.addEventListener(Event.CHANGE, textChanged);
		return tf;
	}

	private function textChanged(evt:Event):void {
		var padding:int = (type == "n") ? 3 : 0;
		base.setWidthHeight(field.width + padding, base.height);
		width = base.width;
		scaleX = scaleY = 1;
		argValue = (type == "n") ? Number(field.text) : field.text;
		if (parent is Block) Block(parent).fixExpressionLayout();
	}

}}
