// Color.as
// John Maloney, August 2009
//
// Color utility methods, such as HSV/RGB conversions.

package {
	
public class Color {

	// Convert hue (0-360), saturation (0-1), and brightness (0-1) to RGB.
	public static function fromHSV(h:Number, s:Number, v:Number):int {
		var r:Number, g:Number, b:Number;
		h = h % 360;
		if (h < 0) h += 360;
		s = Math.max(0, Math.min(s, 1));
		v = Math.max(0, Math.min(v, 1));

		var i:Number = Math.floor(h / 60);
		var f:Number = (h / 60) - i;
		var p:Number = v * (1 - s);
		var q:Number = v * (1 - (s * f));
		var t:Number = v * (1 - (s * (1 - f)));
		if (i == 0) { r = v; g = t; b = p; }
		else if (i == 1) { r = q; g = v; b = p; }
		else if (i == 2) { r = p; g = v; b = t; }
		else if (i == 3) { r = p; g = q; b = v; }
		else if (i == 4) { r = t; g = p; b = v; }
		else if (i == 5) { r = v; g = p; b = q; }
		r = Math.floor(r * 255);
		g = Math.floor(g * 255);
		b = Math.floor(b * 255);
		return (r << 16) | (g << 8) | b;
	}

	// Convert RGB to an array containing the hue, saturation, and brightness.
	public static function rgb2hsv(rgb:Number):Array {
		var h:Number, s:Number, v:Number, x:Number, f:Number, i:Number;
		var r:Number = ((rgb >> 16) & 255) / 255;
		var g:Number = ((rgb >> 8) & 255) / 255;
		var b:Number = (rgb & 255) / 255;
		x = Math.min(Math.min(r, g), b);
		v = Math.max(Math.max(r, g), b);
		if (x == v) return [0, 0, v]; // gray; hue arbitrarily reported as zero
		f = (r == x) ? g - b : ((g == x) ? b - r : r - g);
		i = (r == x) ? 3 : ((g == x) ? 5 : 1);
		h = ((i - (f / (v - x))) * 60) % 360;
		s = (v - x) / v;
		return [h, s, v];
	}

	public static function scaleBrightness(rgb:Number, scale:Number):int {
		var hsv:Array = rgb2hsv(rgb);
		scale = Math.max(0, Math.min(scale, 1));
		return fromHSV(hsv[0], hsv[1], scale * hsv[2]);
	}

}}
